@@ -8,6 +8,7 @@
 
 import sys
 import os
+import pytest
 
 
 class SlaveInteractor:
@@ -33,11 +34,11 @@ class SlaveInteractor:
         slaveinfo = getinfodict()
         self.sendevent("slaveready", slaveinfo=slaveinfo)
 
-    def pytest_sessionfinish(self, __multicall__, exitstatus):
+    @pytest.hookimpl(hookwrapper=True)
+    def pytest_sessionfinish(self, exitstatus):
         self.config.slaveoutput['exitstatus'] = exitstatus
-        res = __multicall__.execute()
+        yield
         self.sendevent("slavefinished", slaveoutput=self.config.slaveoutput)
-        return res
 
     def pytest_collection(self, session):
         self.sendevent("collectionstart")
@@ -88,6 +89,7 @@ class SlaveInteractor:
     def pytest_runtest_logreport(self, report):
         data = serialize_report(report)
         data["item_index"] = self.item_index
+        data["worker_id"] = self.slaveid
         assert self.session.items[self.item_index].nodeid == report.nodeid
         self.sendevent("testreport", data=data)
 
@@ -95,12 +97,43 @@ class SlaveInteractor:
         data = serialize_report(report)
         self.sendevent("collectreport", data=data)
 
+    def pytest_logwarning(self, message, code, nodeid, fslocation):
+        self.sendevent("logwarning", message=message, code=code, nodeid=nodeid,
+                       fslocation=str(fslocation))
+
 
 def serialize_report(rep):
+    def disassembled_report(rep):
+        reprtraceback = rep.longrepr.reprtraceback.__dict__
+        reprcrash = rep.longrepr.reprcrash.__dict__
+
+        new_entries = []
+        for entry in reprtraceback['reprentries']:
+            entry_data = {
+                'type': type(entry).__name__,
+                'data': entry.__dict__,
+            }
+            for key, value in entry_data['data'].items():
+                if hasattr(value, '__dict__'):
+                    entry_data['data'][key] = value.__dict__
+            new_entries.append(entry_data)
+
+        reprtraceback['reprentries'] = new_entries
+
+        return {
+            'reprcrash': reprcrash,
+            'reprtraceback': reprtraceback,
+            'sections': rep.longrepr.sections
+        }
+
     import py
     d = rep.__dict__.copy()
     if hasattr(rep.longrepr, 'toterminal'):
-        d['longrepr'] = str(rep.longrepr)
+        if hasattr(rep.longrepr, 'reprtraceback') \
+                and hasattr(rep.longrepr, 'reprcrash'):
+            d['longrepr'] = disassembled_report(rep)
+        else:
+            d['longrepr'] = str(rep.longrepr)
     else:
         d['longrepr'] = rep.longrepr
     for name in d:
